package xyz.zhouxy.plusone.validator;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.function.Function;
import java.util.function.Predicate;

import cn.hutool.core.exceptions.ValidateException;
import lombok.AllArgsConstructor;
import xyz.zhouxy.plusone.constant.RegexConsts;
import xyz.zhouxy.plusone.util.RegexUtil;

public abstract class BaseValidator2<T> {

    private List<ValidValueHolder<T, ?>> hs = new ArrayList<>();

    protected final <R> ValidValueHolder<T, R> ruleFor(Function<T, R> getter) {
        ValidValueHolder<T, R> validValueHolder = new ValidValueHolder<>(getter);
        hs.add(validValueHolder);
        return validValueHolder;
    }

    public void validate(T obj) {
        for (var holder : hs) {
            var value = holder.getter.apply(obj);
            for (var rule : holder.rules) {
                if (!rule.condition.test(value)) {
                    throw new ValidateException(rule.errMsg);
                }
            }
        }
    }
}

class ValidValueHolder<T, R> {
    Function<T, R> getter;

    List<RuleInfo<Object>> rules = new ArrayList<>();

    public ValidValueHolder(Function<T, R> getter) {
        this.getter = getter;
    }

    private void addRule(Predicate<Object> condition, String errMsg) {
        this.rules.add(new RuleInfo<>(condition, errMsg));
    }

    public ValidValueHolder<T, R> nonNull(String errMsg) {
        addRule(Objects::nonNull, errMsg);
        return this;
    }

    public ValidValueHolder<T, R> nonEmpty(String errMsg) {
        addRule(value -> {
            if (value == null) {
                return false;
            }
            if (value instanceof Collection) {
                return ((Collection<?>) value).isEmpty();
            }
            if (value instanceof String) {
                return ((String) value).isEmpty();
            }
            return false;
        }, errMsg);
        return this;
    }

    public ValidValueHolder<T, R> size(int min, int max, String errMsg) {
        addRule(value -> {
            if (value == null) {
                return false;
            }
            if (value instanceof Collection) {
                int size = ((Collection<?>) value).size();
                return size >= min && size <= max;
            }
            return true;
        }, errMsg);
        return this;
    }

    public ValidValueHolder<T, R> length(int min, int max, String errMsg) {
        addRule(value -> {
            if (value == null) {
                return false;
            }
            if (value instanceof String) {
                int length = ((String) value).length();
                return length >= min && length <= max;
            }
            return true;
        }, errMsg);
        return this;
    }

    public ValidValueHolder<T, R> matches(String regex, String errMsg) {
        addRule(input -> RegexUtil.matches(regex, (String) input), errMsg);
        return this;
    }

    public ValidValueHolder<T, R> matchesOr(String[] regexs, String errMsg) {
        addRule(input -> RegexUtil.matchesOr((String) input, regexs), errMsg);
        return this;
    }

    public ValidValueHolder<T, R> matchesAnd(String[] regexs, String errMsg) {
        addRule(input -> RegexUtil.matchesAnd((String) input, regexs), errMsg);
        return this;
    }

    public ValidValueHolder<T, R> email(String errMsg) {
        return matches(RegexConsts.EMAIL, errMsg);
    }

    @SuppressWarnings("unchecked")
    public ValidValueHolder<T, R> and(Predicate<R> condition, String errMsg) {
        addRule((Predicate<Object>) condition, errMsg);
        return this;
    }

    @AllArgsConstructor
    static final class RuleInfo<R> {
        Predicate<R> condition;
        String errMsg;
    }
}
